/*************************************************************************
  Copyright (c) 1993 - 1997
  Digital Equipment Corporation, Maynard, Mass.

  This software is furnished under a license and may be used and  copied
  only  in  accordance  with  the  terms  of  such  license and with the
  inclusion of the above copyright notice.  This software or  any  other
  copies  thereof may not be provided or otherwise made available to any
  other person.  No title to and ownership of  the  software  is  hereby
  transferred.

  The information in this software is subject to change  without  notice
  and  should  not  be  construed  as  a commitment by DIGITAL Equipment
  Corporation.
 *************************************************************************/


/*++
* ==========
* = rcbr.c =
* ==========
*
* Description:
*   This application demonstrates the API's ability to create and open a PVC
*   using CBR.  This application receives traffic on this PVC, but does not
*   evaluate the link rate based on the PDU arrival rate.  The user must
*   use an ATM analyzer to verify the link rate.  The application notifies
*   the user if a partial packet is detected.  Otherwise a "." is printed on
*   success.  This application is used in conjunction with the api_xmt_cbr
*   executable compiled from xcbr.c.
*
* Authors:
*   TLR - Theodore L. Ross
*   PAE - Paul A. Ely
*
* Modification History:
*   Date          Name     Description
*   22-Sep-1996   TLR      Created.
*   07-Jul-1997   PAE      Modified application to allow non-zero vpi values.
*   21-Jul-1997   PAE      Modified application to allow user-specifiable
*                          packet sizes to 64K bytes.
*
* Arguments:
*   driver name   - The driver service name, e.g. lpa0 or lta0.
*   vpi           - The VPI to use in the PVC.
*   vci           - The VCI to use in the PVC.
*   pkt_size      - The packet size in bytes.  Bound by 0<pkt_size<65536.
*   kbps          - The constant bit rate desired in Kbits/sec.
*   window        - Framing size, typically 0.
*
* Issues:
*   none.
--*/

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <sys/errno.h>
#include <netaal/aal.h>

#define SIZE         65535
#define errtext(n) n

main (int argc, char **argv)
   {
   int                  sock;
   int                  a, i, window;
   int                  cnt = 0;
   unsigned int         pkt_size;
   unsigned long long   bytes = 0;
   unsigned char        buff[SIZE];
   char                 drv_name[32];
   ATM_CONNECTION_ID    conn_id;
   ATM_PARAMS           params;

   /* Prevent garbage array data by clearing out the buff array. */

   for (i = 0; i < 32; i++)
      drv_name[i] = '\0';

   /* The user must pass five command-line arguments. */

   if (argc != 6)
      {
      printf ("Usage: api_rcv_cbr drv_name vpi# vci# pkt_size window\n");
      printf ("\n");
      printf ("   Where drv_name is either lpa or lta and its instance value\n");
      printf ("         vpi# is the vpi of interest\n");
      printf ("         vci# is the vci of interest\n");
      printf ("         pkt_size is the total packet bytes\n");
      printf ("         window is the display window value\n");
      exit(1);
      }

   /* Only lpa and lta driver names are currently supported. */
 
   if (strncmp (argv[1], "lpa", 3) &&
       strncmp (argv[1], "lta", 3) != 0)
      {
      printf ("Unsupported driver service name\n");
      exit (1);
      }

   /* Copy the driver service instance into the conn_id structure. */
   
   strncpy (&drv_name[0], argv[1], (strlen (argv[1]) + 1));

   for (i = 0; i < strlen (&drv_name[0]); i++)
      {
      if ((drv_name[i] >= '0') && (drv_name[i] <= '9'))
         {
         sscanf (&drv_name[i], "%d", &conn_id.device_number);
         break;
         }
      }

   sscanf (argv[2], "%d", &conn_id.vpi);
   sscanf (argv[3], "%d", &conn_id.vci);
   sscanf (argv[4], "%d", &pkt_size);
   sscanf (argv[5], "%d", &window);

   /* Create an AAL socket for DGRAM service. */

   printf ("[ Creating ATM Socket ]\n");
   sock = socket (AF_AAL, SOCK_DGRAM, 0);   
   if (sock == -1)
      {
      printf ("[ socket call failed, error: %d ]\n", errtext (errno));
      exit (1);
      }

   /* Associate the socket with the PVC VCI. */

   printf ("[ Associating Socket with PVC: %s/%d/%d ]\n",
	         &drv_name[0],
            conn_id.vpi,
            conn_id.vci);

   if (ioctl (sock, SIO_ASSOCIATE_PVC, &conn_id) == -1)
      {
      printf ("[ socket ioctl failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Receive all packets until the user selects <CTRL>-C. */

   while (1)
      {
      fd_set readfd;
      i = recv (sock, buff, pkt_size, 0);
      if (i != pkt_size)
         printf ("Received %d bytes out of %d\n", i, pkt_size);

      if (window)
         if ((cnt % window) == (window - 1))
            send (sock, buff, 1, 0);

      cnt++;
      bytes += i;

      if ((cnt % 1) == 0)
         {
         printf (".");
         fflush (stdout);
         }

      if ((cnt % 100) == 0)
         printf ("\n%d, %ld\n", cnt, bytes);
      }

   close (sock);
   }

